package com.tencent.iot.speech.app.asr.file;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.graphics.Color;
import android.net.Uri;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.tencent.cloud.qcloudasrsdk.common.QCloudSourceType;
import com.tencent.cloud.qcloudasrsdk.models.QCloudFileRecognitionParams;
import com.tencent.cloud.qcloudasrsdk.recognizer.QCloudFileRecognizer;
import com.tencent.cloud.qcloudasrsdk.recognizer.QCloudFileRecognizerListener;
import com.tencent.iot.speech.app.DemoConfig;
import com.tencent.iot.speech.app.R;
import com.tencent.iot.speech.app.asr.realtime.MainActivity;
import com.tencent.iot.speech.app.consts.Const;
import com.tencent.iot.speech.app.utils.SharePreferenceUtil;
import com.tencent.iot.speech.asr.listener.MessageListener;
import com.victor.loading.rotate.RotateLoading;

import java.io.IOException;
import java.io.InputStream;

public class FileRecognizeActivity extends AppCompatActivity implements  QCloudFileRecognizerListener, MessageListener {

    private boolean recording = false;

    private QCloudFileRecognizer fileRecognizer;

    private int REQUEST_CODE = 1002;

    private boolean switchToDeviceAuth = false;

    //磁盘读写权限 麦克风录音权限
    String[] permiss = new String[]{Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.RECORD_AUDIO};

    private void showLoading(boolean showLoading) {
        RotateLoading rotateLoading = findViewById(R.id.rotateloading);
        if (showLoading) {
            rotateLoading.start();
        }
        else {
            rotateLoading.stop();
        }
    }
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
//        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        setContentView(R.layout.activity_file_sentence_recognize);


        RotateLoading rotateLoading = findViewById(R.id.rotateloading);
        rotateLoading.setLoadingColor(Color.parseColor("#FF79CD6E"));
        if(!checkMyPermission(permiss)){
            ActivityCompat.requestPermissions(this,permiss, REQUEST_CODE);
        }

        if (fileRecognizer == null) {
            switchToDeviceAuth = SharePreferenceUtil.getBoolean(FileRecognizeActivity.this, Const.APP_CONFIG_FILE, Const.SWITCH_TO_DEVICE_AUTH);
            if (switchToDeviceAuth) {
                fileRecognizer = new QCloudFileRecognizer(this, DemoConfig.appIdForDeviceAuth,
                        DemoConfig.secretIdForDeviceAuth, DemoConfig.secretKeyForDeviceAuth,
                        DemoConfig.serialNumForDeviceAuth, DemoConfig.deviceNumForDeviceAuth, this);
            } else {
                fileRecognizer = new QCloudFileRecognizer(DemoConfig.apppId, DemoConfig.secretId, DemoConfig.secretKey);
            }
            fileRecognizer.setCallback(this);
        }
        System.out.println(Thread.currentThread());

        findViewById(R.id.recognize_ur_btn).setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                try {
                    QCloudFileRecognitionParams params = (QCloudFileRecognitionParams) QCloudFileRecognitionParams.defaultRequestParams();
//                    params.setUrl("http://client-sdk-1255628450.cossh.myqcloud.com/test%20audio/voice_WGVNG_8000.mp3");
                    params.setUrl("https://asr-audio-1256237915.cos.ap-shanghai.myqcloud.com/30s.wav");
                    params.setSourceType(QCloudSourceType.QCloudSourceTypeUrl);
                    params.setFilterDirty(0);// 0 ：默认状态 不过滤脏话 1：过滤脏话
                    params.setFilterModal(0);// 0 ：默认状态 不过滤语气词  1：过滤部分语气词 2:严格过滤
                    params.setFilterPunc(0);// 0 ：默认状态 不过滤句末的句号 1：滤句末的句号
                    params.setConvertNumMode(1);//1：默认状态 根据场景智能转换为阿拉伯数字；0：全部转为中文数字。
//                    params.setHotwordId("");  // 热词id。用于调用对应的热词表，如果在调用语音识别服务时，不进行单独的热词id设置，自动生效默认热词；如果进行了单独的热词id设置，那么将生效单独设置的热词id。

                    long ret = 0;
                    if (switchToDeviceAuth) {
                        ret = fileRecognizer.recognizeForDevice(params);
                    } else {
                        ret = fileRecognizer.recognize(params);
                    }
                    if (ret >= 0) {
                        showLoading(true);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println("exception msg" + e.getMessage());
                } finally {
                }
            }
        });


        findViewById(R.id.recognize_ur_data).setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                InputStream is = null;
                try {
                    AssetManager am = getResources().getAssets();
                    is = am.open("test1.mp3");
//                    is = am.open("8kOne_twopeople.mp3");
                    int length = is.available();
                    byte[] audioData = new byte[length];
                    is.read(audioData);

                    QCloudFileRecognitionParams params = (QCloudFileRecognitionParams) QCloudFileRecognitionParams.defaultRequestParams();
                    params.setData(audioData);
                    params.setSourceType(QCloudSourceType.QCloudSourceTypeData);
                    params.setFilterDirty(0);// 0 ：默认状态 不过滤脏话 1：过滤脏话
                    params.setFilterModal(0);// 0 ：默认状态 不过滤语气词  1：过滤部分语气词 2:严格过滤
                    params.setFilterPunc(0);// 0 ：默认状态 不过滤句末的句号 1：滤句末的句号
                    params.setConvertNumMode(1);//1：默认状态 根据场景智能转换为阿拉伯数字；0：全部转为中文数字。
//                    params.setEngineModelType("8k_0");
//                    params.setSpeakerDiarization(1);
//                    params.setSpeakerNumber(2);

//                    params.setHotwordId("");
                    long ret = 0;
                    if (switchToDeviceAuth) {
                        ret = fileRecognizer.recognizeForDevice(params);
                    } else {
                        ret = fileRecognizer.recognize(params);
                    }
                    if (ret >= 0) {
                        showLoading(true);
                    }
                } catch (IOException e) {
                    showLoading(false);
                    onMessage("录音文件不存在");
                    e.printStackTrace();
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (is != null) is.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        });
    }

    private void updateUI() {
        Button recordButton = findViewById(R.id.recognize_start_record);
        if (recording) {
            recordButton.setText("stopRecord");
        }
        else {
            recordButton.setText("startRecord");
        }
    }

    private boolean checkMyPermission(String[] permiss){
        if(permiss !=null && permiss.length > 0 ){
            for(String per : permiss) {
                if (ContextCompat.checkSelfPermission(this, per) != PackageManager.PERMISSION_GRANTED){
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults){
        if (requestCode == REQUEST_CODE) {
            boolean isAllGranted = true;
            // 判断是否所有的权限都已经授予了
            for (int grant : grantResults) {
                if (grant != PackageManager.PERMISSION_GRANTED) {
                    isAllGranted = false;
                    break;
                }
            }
            if (isAllGranted) {
            } else {
                // 弹出对话框告诉用户需要权限的原因, 并引导用户去应用权限管理中手动打开权限按钮
                openAppDetails();
            }
        }
    }

    /**
     * 打开 APP 的详情设置
     */
    private void openAppDetails() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage("录音需要访问 “外部存储器”，请到 “应用信息 -> 权限” 中授予！");
        builder.setPositiveButton("去手动授权", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Intent intent = new Intent();
                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                intent.addCategory(Intent.CATEGORY_DEFAULT);
                intent.setData(Uri.parse("package:" + getPackageName()));
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
                intent.addFlags(Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS);
                startActivity(intent);
            }
        });
        builder.setNegativeButton("取消", null);
        builder.show();
    }

    @Override
    protected void onStart() {
        super.onStart();
    }


    /*录音文件识别结果回调*/
    @Override
    public void recognizeResult(QCloudFileRecognizer recognizer, final long requestId, String result,int status, Exception exception) {
        if(status==2||status==3||status==-1){ // 3: 无效的音频文件
            showLoading(false);
        }

        TextView textView = findViewById(R.id.recognize_text_view);
        Log.e("recognizeResult","thread id:" + Thread.currentThread().getId() + " name:" + Thread.currentThread().getName());
        if (exception != null) {
            Log.e("recognizeResult","result: " + result + "exception msg" + exception + exception.getLocalizedMessage());
            textView.setText(exception.getLocalizedMessage());
            showLoading(false);
        } else {
            if (status == 3) {
                onMessage("无效的音频文件");
            }
            Log.e("recognizeResult","result: " + result+"---status=="+status);
//            if(!TextUtils.isEmpty(result)){
//
//String pattern = "\\[[^\\]]+\\]";	//中括号内
////                String pattern = "\\([^)]*\\)";//括号内
////String pattern = "\\(.+"
//                result = result.replaceAll(pattern, "");
//            }
            if (result.contains("\"Error\"")) {
                onMessage(result);
            } else {
                textView.setText(result);
            }
        }

    }

    @Override
    public void onMessage(final String msg) {
        runOnUiThread(new Runnable() {
            public void run() {
                Toast.makeText(FileRecognizeActivity.this, msg, Toast.LENGTH_SHORT).show();
            }
        });
    }
}
