package com.tencent.iot.speech.app.tts;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.tencent.iot.speech.app.DemoConfig;
import com.tencent.iot.speech.app.R;
import com.tencent.qcloudtts.LongTextTTS.LongTextTtsController;
import com.tencent.qcloudtts.VoiceLanguage;
import com.tencent.qcloudtts.VoiceSpeed;
import com.tencent.qcloudtts.VoiceType;
import com.tencent.qcloudtts.callback.QCloudPlayerCallback;
import com.tencent.qcloudtts.callback.TtsExceptionHandler;
import com.tencent.qcloudtts.exception.TtsException;
import com.tencent.qcloudtts.exception.TtsNotInitializedException;
import com.tencent.utils.NetworkUtils;
import com.tencent.utils.PermissionHandler;

import java.util.ArrayList;
import java.util.List;

public class LongTextTtsActivity extends AppCompatActivity {
    private int count = 0;
    private static final String TAG = "LongTextTtsActivity";
    private  LongTextTtsController mTtsController;
    private EditText tv_test_tts;
    private int tts_speed = VoiceSpeed.VOICE_SPEED_NORMAL.getNum();
    private int tts_voice = VoiceType.VOICE_TYPE_AFFNITY_FEMALE.getNum();
    private int tts_language = VoiceLanguage.VOICE_LANGUAGE_CHINESE.getNum();

    private TextView textView;

    private Spinner spinner_speed;
    private Spinner spinner_voice;
    private Spinner spinner_language;

    //定义SpinnerItem类型的List数组作为数据源
    private List<SpinnerItem> speedList;
    private List<SpinnerItem> voiceList;
    private List<SpinnerItem> languageList;

    //定义ArrayAdapter适配器作为spinner的数据适配器
    private ArrayAdapter<SpinnerItem> speedAdapter;
    private ArrayAdapter<SpinnerItem> voiceAdapter;
    private ArrayAdapter<SpinnerItem> languageAdapter;

    private AudioManager.OnAudioFocusChangeListener listener;
    private boolean isRegistered = false;
    private NetWorkChangReceiver netWorkChangReceiver;
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        MyCrashHandler handler = new MyCrashHandler();
        Thread.setDefaultUncaughtExceptionHandler(handler);
        //注册网络状态监听广播
        netWorkChangReceiver = new NetWorkChangReceiver();
        IntentFilter filter = new IntentFilter();
        filter.addAction(WifiManager.WIFI_STATE_CHANGED_ACTION);
        filter.addAction(WifiManager.NETWORK_STATE_CHANGED_ACTION);
        filter.addAction(ConnectivityManager.CONNECTIVITY_ACTION);
        registerReceiver(netWorkChangReceiver, filter);
        isRegistered = true;
        setContentView(R.layout.activity_tts);

        tv_test_tts = (EditText) findViewById(R.id.et_test_tts);

        textView = (TextView) findViewById(R.id.et_tts_speed);
        setSpeedSpinner();
        setVoiceSpinner();
        setLanguageSpinner();

        //start按钮
        findViewById(R.id.btn_start).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String ttsText = tv_test_tts.getText().toString();

                if (TextUtils.isEmpty(ttsText)) {
                    ttsText = "2013年11月，习近平在山东农科院召开座谈会时表示，手中有粮，心中不慌。保障粮食安全对中国来说是永恒的课题，任何时候都不能放松。2013年7月，在湖北鄂州东港村育种基地，习近平总书记拔起一棵稻苗察看分蘖情况，夸奖“很壮实”，强调粮食安全要靠自己。高屋建瓴，他的论述为保障粮食安全指明方向";
                }
                if (count % 3 == 0) {
                    ttsText = "北方的风光，千万里冰封冻，千万里雪花飘。远望长城内外，只剩下无边无际白茫茫一片；宽广的黄河上下，顿时失去了滔滔水势。山岭好像银白色的蟒蛇在飞舞，高原上的丘陵好像许多白象在奔跑，它们都想试一试与老天爷比比高。要等到晴天的时候，看红艳艳的阳光和白皑皑的冰雪交相辉映，分外美好。江山如此媚娇，引得无数英雄竞相倾倒。只可惜秦始皇、汉武帝，略差文学才华；唐太宗、宋太祖，稍逊文治功劳。称雄一世的人物，成吉思汗，只知道拉弓射大雕。这些人物全都过去了，数一数能建功立业的英雄人物，还要看今天的人们a";
                }
                else if (count % 3 == 1){
                    ttsText = "接着展现出的是一幅一幅瑰丽变幻的奇景：天姥山隐于云霓明灭之中，引起了诗人探求的想望。诗人进入了梦幻之中，仿佛在月夜清光的照射下，他飞渡过明镜一样的镜湖。明月把他的影子映照在镜湖之上，又送他降落在谢灵运当年曾经歇宿过的地方。他穿上谢灵运当年特制的木屐，登上谢公当年曾经攀登过的石径──青云梯。只见：“半壁见海日，空中闻天鸡。千岩万转路不定，迷花倚石忽已暝。熊咆龙吟殷岩泉，栗深林兮惊层巅。云青青兮欲雨，水澹澹兮生烟。”继飞渡而写山中所见，石径盘旋，深山中光线幽暗，看到海日升空，天鸡高唱，这本是一片曙色；却又于山花迷人、倚石暂憩之中，忽觉暮色降临，旦暮之变何其倏忽。暮色中熊咆龙吟，震响于山谷之间，深林为之战栗，层巅为之惊动。不止有生命的熊与龙以吟、咆表示情感，就连层巅、深林也能战栗、惊动，烟、水、青云都满含阴郁，与诗人的情感，协成一体，形成统一的氛围。前面是浪漫主义地描写天姥山，既高且奇；这里又是浪漫主义地抒情，既深且远。这奇异的境界，已经使人够惊骇的了，但诗人并未到此止步，而诗境却由奇异而转入荒唐，全诗也更进入高潮。";
                }
                else {
                    ttsText = "2013年11月习近平在山东农科院召开座谈会时表示，手中有粮心中不慌。保障粮食安全对中国来说是永恒的课题，任何时候都不能放松。2013年7月，在湖北鄂州东港村育种基地，习近平总书记拔起一棵稻苗察看分蘖情况，夸奖“很壮实”，强调粮食安全要靠自己。高屋建瓴，他的论述为保障粮食安全指明方向";
                }
                count++;
                ttsText = "2013年11月习近平在山东农科院召开座谈会时表示，手中有粮心中不慌。保障粮食安全对中国来说是永恒的课题，任何时候都不能放松。2013年7月，在湖北鄂州东港村育种基地，习近平总书记拔起一棵稻苗察看分蘖情况，夸奖“很壮实”，强调粮食安全要靠自己。高屋建瓴，他的论述为保障粮食安全指明方向。接着展现出的是一幅一幅瑰丽变幻的奇景：天姥山隐于云霓明灭之中，引起了诗人探求的想望。诗人进入了梦幻之中，仿佛在月夜清光的照射下，他飞渡过明镜一样的镜湖。明月把他的影" +
                        "子映照在镜湖之上，又送他降落在谢灵运当年曾经歇宿过的地方。他穿上谢灵运当年特制的木屐，登上谢公当年曾经攀登过的石径──青云梯。只见：“半壁见海日，空中闻天鸡。千岩万转路不定，迷花倚石忽已暝。熊咆龙吟殷岩泉，栗深林兮惊层巅。云青青兮欲雨，水澹澹兮生烟。”继飞渡而写山中所见，石径盘旋，深山中光线幽暗，看到海日升空，天鸡高唱，这本是一片曙色；却又于山花迷人、倚石暂憩之中，忽觉暮色降临，旦暮之变何其倏忽。暮色中熊咆龙吟，震响于山谷之间，深林为之战栗，" +
                        "层巅为之惊动。不止有生命的熊与龙以吟、咆表示情感，就连层巅、深林也能战栗、惊动，烟、水、青云都满含阴郁，与诗人的情感，协成一体，形成统一的氛围。前面是浪漫主义地描写天姥山，既高且奇；这里又是浪漫主义地抒情，既深且远。这奇异的境界，已经使人够惊骇的了，但诗人并未到此止步，而诗境却由奇异而转入荒唐，全诗也更进入高潮。北方的风光，千万里冰封冻，千万里雪花飘。远望长城内外，只剩下无边无际白茫茫一片；宽广的黄河上下，顿时失去了滔滔水势。山岭好像银" +
                        "白色的蟒蛇在飞舞，高原上的丘陵好像许多白象在奔跑，它们都想试一试与老天爷比比高。要等到晴天的时候，看红艳艳的阳光和白皑皑的冰雪交相辉映，分外美好。江山如此媚娇，引得无数英雄竞相倾倒。只可惜秦始皇、汉武帝，略差文学才华；唐太宗、宋太祖，稍逊文治功劳。称雄一世的人物，成吉思汗，只知道拉弓射大雕。这些人物全都过去了，数一数能建功立业的英雄人物，还要看今天的人们a";

                if (mTtsController == null){
                    return;
                }

                //设置语速
                mTtsController.setVoiceSpeed(tts_speed);

                //设置音色
                mTtsController.setVoiceType(tts_voice);

                //设置语言
                mTtsController.setVoiceLanguage(tts_language);

                //设置ProjectId
                mTtsController.setProjectId(0);

                start(ttsText);

            }
        });

        //stop按钮
        findViewById(R.id.btn_stop).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mTtsController!=null){
                    mTtsController.stop();
                }

            }
        });

        //pause按钮
        findViewById(R.id.btn_pause).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mTtsController!=null){
                    mTtsController.pause();
                }

            }
        });

        //resume按钮
        findViewById(R.id.btn_resume).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(NetworkUtils.isNetworkConnected(LongTextTtsActivity.this)){
                    if(mTtsController!=null){
                        mTtsController.resume();
                    }

                }

            }
        });

        //构造LongTextTtsController，支持长文本播放，可暂停/恢复播放。非流式api，故建议文本中第一句话不要设的太长
        mTtsController = new LongTextTtsController();
        /* 在使用云API之前，请前往 腾讯云API密钥页面 申请安全凭证。 安全凭证包括 SecretId 和 SecretKey：
         * SecretId 用于标识 API 调用者身份
         * SecretKey 用于加密签名字符串和服务器端验证签名字符串的密钥。
         */

        //注意：这里只是示例，请根据用户实际申请的 SecretId 和 SecretKey 进行后续操作！
        mTtsController.init(this, Long.valueOf(DemoConfig.apppId)
                , DemoConfig.secretId
                , DemoConfig.secretKey);

        requestAudioFocus(mTtsController);
    }

    //发起tts请求
    private void start(final String ttsText){

        try {
            mTtsController.startTts(ttsText, mTtsExceptionHandler, new QCloudPlayerCallback() {

                //播放开始
                @Override
                public void onTTSPlayStart() {
                    Log.d("tts", "onPlayStart");
                }

                //音频缓冲中
                @Override
                public void onTTSPlayWait() {
                    Log.d("tts", "onPlayWait");
                }

                //缓冲完成，继续播放
                @Override
                public void onTTSPlayResume() {
                    Log.d("tts", "onPlayResume");
                }

                //连续播放下一句
                @Override
                public void onTTSPlayNext() {
                    Log.d("tts", "onPlayNext");
                }

                //播放中止
                @Override
                public void onTTSPlayStop() {
                    Log.d("tts", "onPlayStop");
                }

                //播放结束
                @Override
                public void onTTSPlayEnd() {
                    Log.d("tts", "onPlayEnd");
                }

                //当前播放的字符,当前播放的字符在所在的句子中的下标.
                @Override
                public void onTTSPlayProgress(String currentWord, int currentIndex) {
                    Log.d("tts", "onTTSPlayProgress" + currentWord + currentIndex);
                }
            });
        } catch (TtsNotInitializedException e) {
            Log.e("tts", e.getMessage());
        }
    }

    private final TtsExceptionHandler mTtsExceptionHandler = new TtsExceptionHandler() {
        @Override
        public void onRequestException(TtsException e) {
            Log.e(TAG, "tts onRequestException");
            //网络出错的时候
            mTtsController.pause();
            Toast.makeText(LongTextTtsActivity.this, e.getErrMsg(), Toast.LENGTH_SHORT).show();
        }

    };

    private void setSpeedSpinner() {
        spinner_speed = (Spinner) findViewById(R.id.spinner_speed);

        //为dataList赋值，将下面这些数据添加到数据源中
        speedList = new ArrayList<SpinnerItem>();
        SpinnerItem spinnerItem1=new SpinnerItem(-2, "0.6倍");
        speedList.add(spinnerItem1);
        SpinnerItem spinnerItem2=new SpinnerItem(-1, "0.8倍");
        speedList.add(spinnerItem2);
        SpinnerItem spinnerItem3=new SpinnerItem(0, "正常语速");
        speedList.add(spinnerItem3);
        SpinnerItem spinnerItem4=new SpinnerItem(1, "1.2倍");
        speedList.add(spinnerItem4);
        SpinnerItem spinnerItem5=new SpinnerItem(2, "1.4倍");
        speedList.add(spinnerItem5);

        //为spinner定义适配器，也就是将数据源存入adapter，这里需要三个参数
        speedAdapter = new ArrayAdapter<SpinnerItem>(this, android.R.layout.simple_spinner_item, speedList);

        //为适配器设置下拉列表下拉时的菜单样式。
        speedAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        //为spinner绑定我们定义好的数据适配器
        spinner_speed.setAdapter(speedAdapter);
        spinner_speed.setSelection(2);

        //为spinner绑定监听器，这里我们使用匿名内部类的方式实现监听器
        spinner_speed.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
//                textView.setText("您当前选择的是："+ speedAdapter.getItem(position));
                tts_speed = speedAdapter.getItem(position).getID();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    private void setVoiceSpinner() {
        spinner_voice = (Spinner) findViewById(R.id.spinner_voice);

        //为dataList赋值，将下面这些数据添加到数据源中
        voiceList = new ArrayList<SpinnerItem>();
        SpinnerItem spinnerItem1=new SpinnerItem(0, "亲和女声(默认)");
        voiceList.add(spinnerItem1);
        SpinnerItem spinnerItem2=new SpinnerItem(1, "亲和男声");
        voiceList.add(spinnerItem2);
        SpinnerItem spinnerItem3=new SpinnerItem(2, "成熟男声");
        voiceList.add(spinnerItem3);
        SpinnerItem spinnerItem4=new SpinnerItem(3, "活力男声");
        voiceList.add(spinnerItem4);
        SpinnerItem spinnerItem5=new SpinnerItem(4, "温暖女声");
        voiceList.add(spinnerItem5);
        SpinnerItem spinnerItem6=new SpinnerItem(5, "情感女声");
        voiceList.add(spinnerItem6);
        SpinnerItem spinnerItem7=new SpinnerItem(6, "情感男声");
        voiceList.add(spinnerItem7);

        //为spinner定义适配器，也就是将数据源存入adapter，这里需要三个参数
        voiceAdapter = new ArrayAdapter<SpinnerItem>(this, android.R.layout.simple_spinner_item, voiceList);

        //为适配器设置下拉列表下拉时的菜单样式。
        voiceAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        //为spinner绑定我们定义好的数据适配器
        spinner_voice.setAdapter(voiceAdapter);

        //为spinner绑定监听器，这里我们使用匿名内部类的方式实现监听器
        spinner_voice.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                tts_voice = voiceAdapter.getItem(position).getID();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    private void setLanguageSpinner() {
        spinner_language = (Spinner) findViewById(R.id.spinner_language);
        //为dataList赋值，将下面这些数据添加到数据源中
        languageList = new ArrayList<SpinnerItem>();
        SpinnerItem spinnerItem1=new SpinnerItem(1, "中文");
        languageList.add(spinnerItem1);
        SpinnerItem spinnerItem2=new SpinnerItem(2, "英文");
        languageList.add(spinnerItem2);

        //为spinner定义适配器，也就是将数据源存入adapter，这里需要三个参数
        languageAdapter = new ArrayAdapter<SpinnerItem>(this, android.R.layout.simple_spinner_item, languageList);

        //为适配器设置下拉列表下拉时的菜单样式。
        languageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        //为spinner绑定我们定义好的数据适配器
        spinner_language.setAdapter(languageAdapter);

        //为spinner绑定监听器，这里我们使用匿名内部类的方式实现监听器
        spinner_language.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                tts_language = languageAdapter.getItem(position).getID();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    private void requestAudioFocus(@NonNull final LongTextTtsController ttsController) {
        //初始化audio mananger
        listener = new AudioManager.OnAudioFocusChangeListener() {
            @Override
            public void onAudioFocusChange(int focusChange) {
                if (focusChange == AudioManager.AUDIOFOCUS_LOSS) {
                    //丢失焦点，直接
                    mTtsController.stop();
                } else if (focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT) {
                    //丢失焦点，但是马上又能恢复
                } else if (focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK) {
                    //降低音量
                } else if (focusChange == AudioManager.AUDIOFOCUS_GAIN) {
                    //获得了音频焦点
                }
            }
        };

        //设置listener
        AudioManager am = (AudioManager) getApplicationContext().getSystemService(Context.AUDIO_SERVICE);
        if (am != null) {
            am.requestAudioFocus(listener, AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);
        }
    }

    private void abandonAudioFocus() {
        AudioManager am = (AudioManager) getApplicationContext().getSystemService(Context.AUDIO_SERVICE);
        if (am != null) {
            am.abandonAudioFocus(listener);
        }
    }

    @Override
    protected void onDestroy() {
        //解绑
        if (isRegistered) {
            unregisterReceiver(netWorkChangReceiver);
        }
        boolean finished = this.isFinishing();
        if (mTtsController != null) {
            mTtsController.stop();
            mTtsController = null;
        }
        super.onDestroy();
        abandonAudioFocus();
    }


    @Override
    protected void onPause() {
        super.onPause();

    }

    @Override
    public void finish() {
        super.finish();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        mPermissionHandler.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    private PermissionHandler mPermissionHandler = new PermissionHandler(this) {
        @Override
        protected String[] onGetPermissions() {
            return new String[]{
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.INTERNET
            };
        }

        @Override
        protected void onPreRequestPermissions(String[] permissions) {/* notify user */}

        @Override
        protected void onPermissionsDecline(String[] permissions) {/* notify user */}

        @Override
        protected void onAllPermissionGranted() {/* notify user */}
    };

    public  class NetWorkChangReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {

            //1.1 判断当前网络状态
            String networkType = NetworkUtils.getNetworkTypeDesc(context);
            // 监听网络连接，包括wifi和移动数据的打开和关闭,以及连接上可用的连接都会接到监听
            if (ConnectivityManager.CONNECTIVITY_ACTION.equals(intent.getAction())) {
                //获取联网状态的NetworkInfo对象
                NetworkInfo info = intent.getParcelableExtra(ConnectivityManager.EXTRA_NETWORK_INFO);
                if (info != null) {
                    //如果当前的网络连接成功并且网络连接可用
                    if (NetworkInfo.State.CONNECTED == info.getState() && info.isAvailable()) {
                        if (info.getType() == ConnectivityManager.TYPE_WIFI
                                || info.getType() == ConnectivityManager.TYPE_MOBILE
                                ||info.getType() ==ConnectivityManager.TYPE_ETHERNET) {
                            Log.i(TAG, networkType + "连上");
//                            mTtsController.resume();
                        }
                    } else {

                        if(mTtsController!=null){
                            mTtsController.pause();
                        }
                        Log.i(TAG, networkType + "断开");
                    }
                }
            }
        }
    }


}
